using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace ThumbViewer
{
    public partial class SearchForm : Form
    {
        public String m_strPath = "";
        private bool m_bCancel = false;

        public SearchForm()
        {
            InitializeComponent();
        }

        private void OnLoad(object sender, EventArgs e)
        {
            String strPath = System.Environment.GetFolderPath(System.Environment.SpecialFolder.System);
            strPath = Path.GetPathRoot(strPath);

            this.comboPath.Items.Add(strPath);
            this.comboPath.SelectedIndex = 0;
        }

        private void OnSelectionChanged(object sender, EventArgs e)
        {
            int nIndex = this.listboxResult.SelectedIndex;
            if (nIndex == 0 || nIndex == this.listboxResult.Items.Count - 1)
                this.buttonSelect.Enabled = false;
            else
            {
                String strPath = this.listboxResult.SelectedItem.ToString();
                if (strPath.IndexOf("Cancelled by user") == -1)
                    this.buttonSelect.Enabled = true;
            }
        }

        private void OnPathDropDown(object sender, EventArgs e)
        {
            foreach (String strItem in this.comboPath.Items)
                if (strItem == this.comboPath.Text)
                    return;

            this.comboPath.Items.Add(comboPath.Text);
        }

        private void OnButtonBrowse(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            fbd.ShowNewFolderButton = false;
            fbd.Description = "Browser for Folder where Pictures and database file, created from these Pictures " +
                "by Explorer (Thumbs.db), can be located:";

            if (DialogResult.OK == fbd.ShowDialog())
            {
                this.comboPath.Text = fbd.SelectedPath;
                this.comboPath.Items.Add(fbd.SelectedPath);
            }
        }

        private void OnButtonStart(object sender, EventArgs e)
        {
            this.buttonStart.Enabled = false;
            this.buttonStop.Enabled = true;
            m_bCancel = false;
            try
            {
                using (ThumbViewer.FileSystemEnumerator fse = new ThumbViewer.FileSystemEnumerator(comboPath.Text, "Thumbs.db", checkInclude.Checked))
                {
                    IEnumerator<FileInfo> ien = fse.Matches().GetEnumerator();
                    ien.Dispose();

                    this.listboxResult.Items.Clear();
                    this.listboxResult.Items.Add("Size             Modified  Full Path");
                    foreach (FileInfo fi in fse.Matches())
                    {
                        this.listboxResult.Items.Add(string.Format("{0,-14:#,##0}{1,12:d} {2}", fi.Length, fi.LastWriteTime, fi.FullName));
                        Application.DoEvents();
                        if (m_bCancel)
                        {
                            listboxResult.Items.Add("Cancelled by user");
                            break;
                        }
                    }

                    this.listboxResult.Items.Add(string.Format("Found {0} matches", listboxResult.Items.Count - 1));
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                  string.Format("Exception {0}: {1}", ex.GetType().Name, ex.Message),
                  "Thumbs Cleaner",
                  MessageBoxButtons.OK,
                  MessageBoxIcon.Error
                  );
            }
            this.buttonStart.Enabled = true;
            this.buttonStop.Enabled = false;
        }

        private void OnButtonStop(object sender, EventArgs e)
        {
            m_bCancel = true;
            this.buttonStop.Enabled = false;
        }

        private void OnButtonSelect(object sender, EventArgs e)
        {
            m_strPath = this.listboxResult.SelectedItem.ToString();
            m_strPath = m_strPath.Substring(26);
        }
    }
}