using System;
using System.Drawing;
using System.Drawing.Text;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace ThumbViewer
{
    public struct ButtonColors
    {
        public Color BorderColor1;
        public Color BorderColor2;

        public Color FillColor1;
        public Color FillColor2;
        public Color FillColor3;
        public Color FillColor4;
    }

    public class VistaButton : VistaButtonBase
    {
        public ButtonColors NormalColors;
        public ButtonColors HoverColors;
        public ButtonColors PressedColors;

        public VistaButton()
            : base()
        {
            NormalColors.BorderColor1 = Color.FromArgb(142, 143, 143);
            NormalColors.BorderColor2 = Color.FromArgb(142, 143, 143);

            NormalColors.FillColor1 = Color.FromArgb(240, 240, 240);
            NormalColors.FillColor2 = Color.FromArgb(235, 235, 235);
            NormalColors.FillColor3 = Color.FromArgb(224, 224, 224);
            NormalColors.FillColor4 = Color.FromArgb(215, 215, 215);

            HoverColors = NormalColors;
            PressedColors = NormalColors;

            HoverColors.BorderColor1 = Color.FromArgb(246, 241, 228);
            HoverColors.BorderColor2 = Color.FromArgb(228, 218, 197);
            HoverColors.FillColor1 = Color.FromArgb(255, 254, 227);
            HoverColors.FillColor2 = Color.FromArgb(255, 231, 151);
            HoverColors.FillColor3 = Color.FromArgb(254, 214, 101);
            HoverColors.FillColor4 = Color.FromArgb(255, 239, 179);

            //////////////////////////////////////////////////////////////
            PressedColors.BorderColor1 = Color.FromArgb(60, 127, 177);
            PressedColors.BorderColor2 = Color.FromArgb(60, 127, 177);
            PressedColors.FillColor1 = Color.FromArgb(221, 242, 252);
            PressedColors.FillColor2 = Color.FromArgb(215, 239, 252);
            PressedColors.FillColor3 = Color.FromArgb(189, 228, 250);
            PressedColors.FillColor4 = Color.FromArgb(171, 221, 248);
        }

        protected override void OnPaint(System.Windows.Forms.PaintEventArgs pevent)
        {
            Graphics g = pevent.Graphics;
            ButtonColors clr = NormalColors;

            if (state == States.MouseOver)
                clr = HoverColors;
            if (state == States.Pushed)
                clr = PressedColors;

            Rectangle baseRect = new Rectangle(0, 0, this.Width, this.Height);
            Rectangle rect = baseRect;
            //////////////////////////////////////////////////////////////////

            LinearGradientBrush innerBorderFill = new LinearGradientBrush(new Point(0, 0), new Point(0, this.Height), Color.White, Color.White);
            ColorBlend borderBlend = new ColorBlend();
            int alpha = 220;
            borderBlend.Colors = new Color[] { Color.FromArgb(alpha, clr.FillColor1), Color.FromArgb(alpha, clr.FillColor2), Color.FromArgb(alpha, clr.FillColor3), Color.FromArgb(alpha, clr.FillColor4) };
            borderBlend.Positions = new float[] { 0, 0.5F, 0.5F + 0.00000001F, 1 };
            innerBorderFill.InterpolationColors = borderBlend;

            pevent.Graphics.FillRectangle(innerBorderFill, rect);

            innerBorderFill.Dispose();
            //////////////////////////////////////////////////////////////////
            LinearGradientBrush borderFill = new LinearGradientBrush(new Point(0, 0), new Point(0, this.Height), clr.BorderColor1, clr.BorderColor2);
            Pen border = new Pen(borderFill);
            pevent.Graphics.DrawPath(border, GetPath(this.Width - 1, this.Height - 1));
            border.Dispose();
            borderFill.Dispose();
            //////////////////////////////////////////////////////////////////
            // Fill upper part
            rect.Inflate(-2, -2);
            rect.Height = rect.Height / 2;

            LinearGradientBrush innerFill = new LinearGradientBrush(rect, clr.FillColor1, clr.FillColor2, LinearGradientMode.Vertical);
            pevent.Graphics.FillRectangle(innerFill, rect);
            innerFill.Dispose();
            //////////////////////////////////////////////////////////////////
            // Fill lower part
            Rectangle rectangle1 = new Rectangle(0, this.Height, this.Width, this.Height / 2);

            int radius = (int)Math.Sqrt((double)((rectangle1.Width * rectangle1.Width) + (rectangle1.Height * rectangle1.Height)));
            GraphicsPath path2 = new GraphicsPath();
            path2.AddEllipse(rectangle1.X - (radius - rectangle1.Width), rectangle1.Y - ((radius - rectangle1.Height) / 2), radius, radius);

            SolidBrush brush1 = new SolidBrush(clr.FillColor3);
            rect.Y += rect.Height;
            pevent.Graphics.FillRectangle(brush1, rect);
            brush1.Dispose();

            PathGradientBrush brush3 = new PathGradientBrush(path2);
            brush3.FocusScales = new PointF(0.2F, 0.2F);
            brush3.CenterColor = clr.FillColor4;
            brush3.SurroundColors = new Color[] { clr.FillColor3 };

            pevent.Graphics.FillRectangle(brush3, rect);
            brush3.Dispose();
            path2.Dispose();
            //////////////////////////////////////////////////////////////////
            StringFormat sf = getAlignment();
            sf.Trimming = StringTrimming.EllipsisCharacter;
            sf.HotkeyPrefix = HotkeyPrefix.Show;

            RectangleF layoutRect = baseRect;
            layoutRect.Inflate(-4, -4);

            if (this.Enabled)
            {
                SolidBrush textBrush = new SolidBrush(this.ForeColor);
                pevent.Graphics.DrawString(this.Text, this.Font, textBrush, layoutRect, sf);
                textBrush.Dispose();
            }
            else
            {
                SolidBrush textBrush = new SolidBrush(SystemColors.GrayText);
                pevent.Graphics.DrawString(this.Text, this.Font, textBrush, layoutRect, sf);
                textBrush.Dispose();
            }
            //////////////////////////////////////////////////////////////////

            if (this.Focused)
            {
                Rectangle focusRect = baseRect;
                focusRect.Inflate(-3, -3);
                ControlPaint.DrawFocusRectangle(pevent.Graphics, focusRect);
            }
        }
    }

    public class VistaButtonBase : System.Windows.Forms.Button
    {
        public VistaButtonBase()
            : base()
        {
            this.DoubleBuffered = true;
            this.FlatStyle = FlatStyle.Flat;
        }

        public enum States
        {
            Normal,
            MouseOver,
            Pushed
        }

        protected States state = States.Normal;

        protected GraphicsPath GetPath(int width, int height)
        {
            int X = width;
            int Y = height;

            Rectangle[] rects0;
            Rectangle[] rects1;

            rects0 = new Rectangle[2];
            rects0[0] = new Rectangle(2, 4, 2, Y - 8);
            rects0[1] = new Rectangle(X - 4, 4, 2, Y - 8);

            rects1 = new Rectangle[8];
            rects1[0] = new Rectangle(2, 1, 2, 2);
            rects1[1] = new Rectangle(1, 2, 2, 2);
            rects1[2] = new Rectangle(X - 4, 1, 2, 2);
            rects1[3] = new Rectangle(X - 3, 2, 2, 2);
            rects1[4] = new Rectangle(2, Y - 3, 2, 2);
            rects1[5] = new Rectangle(1, Y - 4, 2, 2);
            rects1[6] = new Rectangle(X - 4, Y - 3, 2, 2);
            rects1[7] = new Rectangle(X - 3, Y - 4, 2, 2);

            Point[] points = {
								 new Point(1, 0),
								 new Point(X-1, 0),
								 new Point(X-1, 1),
								 new Point(X, 1),
								 new Point(X, Y-1),
								 new Point(X-1, Y-1),
								 new Point(X-1, Y),
								 new Point(1, Y),
								 new Point(1, Y-1),
								 new Point(0, Y-1),
								 new Point(0, 1),
								 new Point(1, 1)};

            GraphicsPath path = new GraphicsPath();
            path.AddLines(points);

            return path;
        }

        private void CreateRegion()
        {
            this.Region = new Region(GetPath(this.Width, this.Height));
        }

        protected override void OnParentChanged(EventArgs e)
        {
            if (Parent == null) return;

            CreateRegion();
            base.OnParentChanged(e);
        }
        protected override void OnMouseEnter(EventArgs e)
        {
            state = States.MouseOver;
            this.Invalidate();
            base.OnMouseEnter(e);
        }
        protected override void OnMouseLeave(EventArgs e)
        {
            state = States.Normal;
            this.Invalidate();
            base.OnMouseLeave(e);
        }
        protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) != MouseButtons.Left) return;

            state = States.Pushed;

            this.Invalidate();
            base.OnMouseDown(e);
        }
        protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
                state = States.MouseOver;
            this.Invalidate();
            base.OnMouseUp(e);
        }
        protected override void OnEnter(System.EventArgs e)
        {
            this.Invalidate();
            base.OnEnter(e);
        }
        protected override void OnLeave(System.EventArgs e)
        {
            this.Invalidate();
            base.OnLeave(e);
        }
        protected override void OnClick(System.EventArgs e)
        {
            if (state == States.Pushed)
            {
                state = States.Normal;
                this.Invalidate();
            }
            base.OnClick(e);
        }
        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged(e);

            CreateRegion();
            this.Invalidate();
        }

        protected override void OnPaint(System.Windows.Forms.PaintEventArgs pevent)
        {
            Rectangle rect = new Rectangle(0, 0, this.Width, this.Height);
            //////////////////////////////////////////////////////////////////

            LinearGradientBrush innerBorderFill = new LinearGradientBrush(new Point(0, 0), new Point(0, this.Height), Color.White, Color.White);
            ColorBlend borderBlend = new ColorBlend();
            borderBlend.Colors = new Color[] { Color.FromArgb(255, 255, 251), Color.FromArgb(255, 249, 227), Color.FromArgb(255, 242, 201), Color.FromArgb(255, 231, 150) };
            borderBlend.Positions = new float[] { 0, 0.5F, 0.5F + 0.00000001F, 1 };
            innerBorderFill.InterpolationColors = borderBlend;
            innerBorderFill.GammaCorrection = false;

            pevent.Graphics.FillRectangle(innerBorderFill, rect);

            innerBorderFill.Dispose();
            //////////////////////////////////////////////////////////////////
            LinearGradientBrush borderFill = new LinearGradientBrush(new Point(0, 0), new Point(0, this.Height), Color.FromArgb(246, 241, 228), Color.FromArgb(228, 218, 197));
            Pen border = new Pen(borderFill);
            pevent.Graphics.DrawPath(border, GetPath(this.Width - 1, this.Height - 1));
            border.Dispose();
            borderFill.Dispose();
            //////////////////////////////////////////////////////////////////
            // Fill upper part
            rect.Inflate(-2, -2);
            rect.Height = rect.Height / 2;

            LinearGradientBrush innerFill = new LinearGradientBrush(rect, Color.FromArgb(255, 254, 227), Color.FromArgb(255, 231, 151), LinearGradientMode.Vertical);
            pevent.Graphics.FillRectangle(innerFill, rect);
            innerFill.Dispose();
            //////////////////////////////////////////////////////////////////
            // Fill lower part
            Rectangle rectangle1 = new Rectangle(0, this.Height, this.Width, this.Height / 2);

            int radius = (int)Math.Sqrt((double)((rectangle1.Width * rectangle1.Width) + (rectangle1.Height * rectangle1.Height)));
            GraphicsPath path2 = new GraphicsPath();
            path2.AddEllipse(rectangle1.X - (radius - rectangle1.Width), rectangle1.Y - ((radius - rectangle1.Height) / 2), radius, radius);

            SolidBrush brush1 = new SolidBrush(Color.FromArgb(254, 214, 101));
            rect.Y += rect.Height;
            pevent.Graphics.FillRectangle(brush1, rect);
            brush1.Dispose();

            PathGradientBrush brush3 = new PathGradientBrush(path2);
            brush3.CenterColor = Color.FromArgb(255, 239, 179);
            brush3.SurroundColors = new Color[] { Color.FromArgb(254, 214, 101) };

            pevent.Graphics.FillRectangle(brush3, rect);
            brush3.Dispose();
            path2.Dispose();
            //////////////////////////////////////////////////////////////////
            StringFormat sf = getAlignment();
            sf.Trimming = StringTrimming.EllipsisCharacter;

            RectangleF layoutRect = new RectangleF(4, 4, Width - 4, Height - 4);

            SolidBrush textBrush = new SolidBrush(this.ForeColor);
            pevent.Graphics.DrawString(this.Text, this.Font, textBrush, layoutRect, sf);
            textBrush.Dispose();
            //////////////////////////////////////////////////////////////////

            if (this.Focused)
            {
                Rectangle focusRect = new Rectangle(3, 3, Width - 3, Height - 3);
                ControlPaint.DrawFocusRectangle(pevent.Graphics, focusRect);
            }
        }

        protected StringFormat getAlignment()
        {
            StringFormat sf = new StringFormat();

            switch (this.TextAlign)
            {
                case ContentAlignment.BottomCenter:
                    sf.LineAlignment = StringAlignment.Far;
                    sf.Alignment = StringAlignment.Center;
                    break;

                case ContentAlignment.BottomLeft:
                    sf.LineAlignment = StringAlignment.Far;
                    sf.Alignment = StringAlignment.Near;
                    break;

                case ContentAlignment.BottomRight:
                    sf.LineAlignment = StringAlignment.Far;
                    sf.Alignment = StringAlignment.Far;
                    break;


                case ContentAlignment.MiddleCenter:
                    sf.LineAlignment = StringAlignment.Center;
                    sf.Alignment = StringAlignment.Center;
                    break;

                case ContentAlignment.MiddleLeft:
                    sf.LineAlignment = StringAlignment.Center;
                    sf.Alignment = StringAlignment.Near;
                    break;

                case ContentAlignment.MiddleRight:
                    sf.LineAlignment = StringAlignment.Center;
                    sf.Alignment = StringAlignment.Far;
                    break;


                case ContentAlignment.TopCenter:
                    sf.LineAlignment = StringAlignment.Near;
                    sf.Alignment = StringAlignment.Center;
                    break;

                case ContentAlignment.TopLeft:
                    sf.LineAlignment = StringAlignment.Near;
                    sf.Alignment = StringAlignment.Near;
                    break;

                case ContentAlignment.TopRight:
                    sf.LineAlignment = StringAlignment.Near;
                    sf.Alignment = StringAlignment.Far;
                    break;
            }

            return sf;
        }

    }
}
